####
## Phase C - step 8. Predict extra characteristics
####

##### 0. initialize

library(foreign)
library(rgdal)
library(plyr)
library(raster)
library(stargazer)
library(MASS)
library(ordinal)
library(nnet)
library(questionr)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

set.seed(1) #For population 1 we use set.seed(1), for population 2 we use set.seed(2) etc. 

sink("Output/populations/synth_pop_1/StepC8.txt")

#from https://stackoverflow.com/questions/30195718/stargazer-save-to-file-dont-show-in-console
mod_stargazer <- function(output.file, ...) {
  output <- capture.output(stargazer(...))
  cat(paste(output, collapse = "\n"), "\n", file=output.file)
}

getmode <- function(v) {
  uniqv <- unique(v)
  if (sum(!is.na(uniqv)) > 0) {
    uniqv <- uniqv[!is.na(uniqv)]
  }
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

#from https://www.r-bloggers.com/how-to-multinomial-regression-models-in-r/
#adapted so that it can also handle binomial cases 
predictMNL <- function(model, newdata) {
  # Only works for neural network models
  if (is.element("nnet",class(model))) {
    # Calculate the individual and cumulative probabilities
    probs <- predict(model,newdata,"probs")
    if (is.null(dim(probs))) {
      cum.probs <- probs
    } else {
      cum.probs <- t(apply(probs,1,cumsum))
    }
    # Draw random values
    vals <- runif(nrow(newdata))
    
    # Join cumulative probabilities and random draws
    tmp <- cbind(cum.probs,vals)
    
    # For each row, get choice index.
    if (is.null(dim(probs))) {
      k <- 1
    } else {
      k <- ncol(probs)
    }
    ids <- 1 + apply(tmp,1,function(x) length(which(x[1:k] < x[k+1])))
    # Return the values
    return(ids)
  }
}


##### 1. load and merge data

## DHS
dhs_ind <- read.dta("Data/DHS data/Generated data/DHS_ind_recode.dta")
#dhs_ind <- dhs_ind[dhs_ind$region == "oshikoto",]
dim(dhs_ind)  #3136 24
summary(dhs_ind)

dhs_ind$hh_weight <- dhs_ind$hh_weight/100000

dhs_ind$u5 <- ifelse(dhs_ind$age %in% c("0", "1 - 4"), 1, 0)
dhs_ind <- ddply(dhs_ind,.(hhid), transform, 
                 any_u5 = max(u5))

summary(dhs_ind$any_u5)

dhs_ind$noeduc <- ifelse(dhs_ind$education =="No formal", 1, 0)
dhs_ind$noeduc <- as.factor(dhs_ind$noeduc)
levels(dhs_ind$noeduc) <- c("Any formal education","No formal education")

#get factor levels of education corresponding with the synthetic population
dhs_ind$education[dhs_ind$education %in% "Don't know"] <- NA
dhs_ind$education <- as.numeric(dhs_ind$education)
dhs_ind$education <- as.factor(dhs_ind$education)
levels(dhs_ind$education) <- c("No formal", "Incomplete primary", "Complete primary", "Complete secondary", "Complete tertiary")

dhs_ind$educ_hhhead[dhs_ind$educ_hhhead %in% "Don't know"] <- NA
dhs_ind$educ_hhhead <- as.numeric(dhs_ind$educ_hhhead)
dhs_ind$educ_hhhead <- as.factor(dhs_ind$educ_hhhead)
levels(dhs_ind$educ_hhhead) <- c("No formal", "Incomplete primary", "Complete primary", "Complete secondary", "Complete tertiary")

dhs_ind$noeduc_hhhead <- ifelse(dhs_ind$educ_hhhead =="No formal", 1, 0)
dhs_ind[is.na(dhs_ind$educ_hhhead),]$noeduc_hhhead <- NA
dhs_ind$noeduc_hhhead <- as.factor(dhs_ind$noeduc_hhhead)
levels(dhs_ind$noeduc_hhhead) <- c("Any formal education","No formal education")

dhs_ind$age_hhhead_temp <- dhs_ind$age_hhhead
dhs_ind$age_hhhead <- as.numeric(dhs_ind$age_hhhead)
dhs_ind[dhs_ind$age_hhhead %in% c(1:7),]$age_hhhead <- 1
dhs_ind[dhs_ind$age_hhhead %in% c(8:11),]$age_hhhead <- 2
dhs_ind[dhs_ind$age_hhhead %in% c(12:16),]$age_hhhead <- 3
dhs_ind$age_hhhead <- as.factor(dhs_ind$age_hhhead)
levels(dhs_ind$age_hhhead) <- c("under 30", "30-49", "50 plus")
dhs_ind <- dhs_ind[,!names(dhs_ind) %in% c("age_hhhead_temp")]

#allocated synthetic population - remember: each population stored in seperate folder "synth_pop_x"
synthpop_loc <- readOGR("Output/populations/synth_pop_1/Oshikoto_synthpop_loc.shp")
dim(synthpop_loc@data) #37156    43

synthpop_loc@data <- synthpop_loc@data[,names(synthpop_loc@data) %in%
                                     c("long_ct", "lat_act", "EA", "Cnstt_1","uniq_id", "grid_nr", 
                                       "hhid", "hhsize", "urbn_rr", "sx_hhhd", "ag_hhhd", 
                                       "fuel", "structr", "toilet", "water", "space", "slum", "any_u5", "edc_hhh")]
 
names(synthpop_loc@data) <- c("long", "lat", "EA", "constituency", "uniq_id", "grid_nr", 
                            "hhid", "hhsize","urban_rural", "sex_hhhead", "age_hhhead", 
                            "fuel", "structure", "toilet", "water", "space", "slum", "any_u5", "educ_hhhead")

synthpop_loc$fuel <- as.factor(synthpop_loc$fuel)
levels(synthpop_loc$fuel) <- levels(dhs_ind$fuel)
synthpop_loc$structure <- as.factor(synthpop_loc$structure)
levels(synthpop_loc$structure) <- levels(dhs_ind$structure)
synthpop_loc$toilet <- as.factor(synthpop_loc$toilet)
levels(synthpop_loc$toilet) <- levels(dhs_ind$toilet)
synthpop_loc$water <- as.factor(synthpop_loc$water)
levels(synthpop_loc$water) <- levels(dhs_ind$water)
synthpop_loc$space <- as.factor(synthpop_loc$space)
levels(synthpop_loc$space) <- levels(dhs_ind$space)
synthpop_loc$slum <- as.factor(synthpop_loc$slum)
levels(synthpop_loc$slum) <- levels(dhs_ind$slum)

synthpop_loc$noeduc_hhhead <- ifelse(synthpop_loc$educ_hhhead =="No formal", 1, 0)
synthpop_loc$noeduc_hhhead <- as.factor(synthpop_loc$noeduc_hhhead)
levels(synthpop_loc$noeduc_hhhead) <- c("Any formal education","No formal education")

synthpop_loc$age_hhhead_temp <- synthpop_loc$age_hhhead
synthpop_loc$age_hhhead <- as.numeric(synthpop_loc$age_hhhead)
# Be careful, whereas in dhs_ind as.numeric gives counts starting at age cat 0, 
# here we get counts starting at age cat 10-14. so under thirty is 1:4 instead of 1:7 etc. 
synthpop_loc@data[synthpop_loc@data$age_hhhead %in% c(1:4),]$age_hhhead <- 1
synthpop_loc@data[synthpop_loc@data$age_hhhead %in% c(5:8),]$age_hhhead <- 2
synthpop_loc@data[synthpop_loc@data$age_hhhead %in% c(9:13),]$age_hhhead <- 3
synthpop_loc$age_hhhead <- as.factor(synthpop_loc$age_hhhead)
levels(synthpop_loc$age_hhhead) <- c("under 30", "30-49", "50 plus")
synthpop_loc <- synthpop_loc[,!names(synthpop_loc) %in% c("age_hhhead_temp")]

#add household members
synthpop_ind <- readRDS(file = "Output/populations/synth_pop_1/nam_synth_person.rds")
synthpop_ind <- synthpop_ind[, names(synthpop_ind)%in% 
                               c("hhid", "pid", "relationship", "age", "sex", "education", "u5")]
dim(synthpop_ind) # 188997 7

synthpop_hh_loc <- data.frame(synthpop_loc@data)
synthpop_ind_loc <- ddply(merge(synthpop_ind, synthpop_hh_loc), .(hhid))

synthpop_ind_loc$education <- as.numeric(synthpop_ind_loc$education)
synthpop_ind_loc$education <- as.factor(synthpop_ind_loc$education)
levels(synthpop_ind_loc$education) <- c("No formal", "Incomplete primary", "Complete primary", "Complete secondary", "Complete tertiary")

synthpop_ind_loc$noeduc <- ifelse(synthpop_ind_loc$education =="No formal", 1, 0)
synthpop_ind_loc$noeduc <- as.factor(synthpop_ind_loc$noeduc)
levels(synthpop_ind_loc$noeduc) <- levels(dhs_ind$noeduc)

synthpop_loc@data <- data.frame(synthpop_loc@data[, names(synthpop_loc@data) %in% "uniq_id"])

rm(synthpop_ind, synthpop_hh_loc)
gc()

##### 2. Predict wealth, vaccination, and womens contraception. 

## wealth 
model_wealth <- multinom(wealth_index ~ urban_rural + age_hhhead + sex_hhhead + educ_hhhead +
                                   water + toilet + space + structure + fuel, 
                    data = dhs_ind, weights = hh_weight, subset = (relationship %in% c("head")))

mod_stargazer("output/populations/synth_pop_1/wealth.txt", model_wealth, header=FALSE, type = "text")

loglik_model_wealth <- nnet:::logLik.multinom(model_wealth)
model_wealth0 <- multinom(wealth_index ~ 1, data = dhs_ind, weights = hh_weight, subset = (relationship %in% c("head")))
loglik_model_wealth0 <- nnet:::logLik.multinom(model_wealth0)
mfr2_model_wealth <- as.numeric(1 - loglik_model_wealth/loglik_model_wealth0)
mfr2_model_wealth

synthpop_ind_loc$wealth_index <- predictMNL(model_wealth,synthpop_ind_loc)
synthpop_ind_loc$wealth_index <- ifelse(synthpop_ind_loc$relationship %in% c("head"), synthpop_ind_loc$wealth_index, NA)
synthpop_ind_loc <- ddply(synthpop_ind_loc,.(hhid), transform, 
                   wealth_index = getmode(wealth_index))
synthpop_ind_loc$wealth_index <- as.factor(synthpop_ind_loc$wealth_index)
levels(synthpop_ind_loc$wealth_index) <- levels(dhs_ind$wealth_index)

#compare
prop.table(table(synthpop_ind_loc$wealth_index))
prop.table(wtd.table(x = dhs_ind[dhs_ind$region == "oshikoto",]$wealth_index, weights = dhs_ind[dhs_ind$region == "oshikoto",]$hh_weight))
#seems to underestimate high wealth individuals, but this may be due to the composition of the population compared to DHS, 
#for example in our population the proportion with unimproved water source much higher than in DHS.

## contraception
dhs_ind$elig_women <- ifelse(dhs_ind$age %in% c("15 - 19", "20 - 24", "25 - 29", "30 - 34", 
                                                "35 - 39", "40 - 44", "45 - 49") & 
                          dhs_ind$sex %in% c("Female"),1,0)
synthpop_ind_loc$elig_women <- ifelse(synthpop_ind_loc$age %in% c("15 - 19", "20 - 24", "25 - 29", "30 - 34", 
                                                                  "35 - 39", "40 - 44", "45 - 49") & 
                                      synthpop_ind_loc$sex %in% c("Female"),1,0)

#We do not want R to try to estimate coefficients for levels of age which do not exist for these women. 
#Solve this in a 'stupid' way by making temp variable specific to this age group

dhs_ind$age_ew <- dhs_ind$age
dhs_ind[dhs_ind$elig_women == 0,]$age_ew <- NA
dhs_ind$age_ew <- as.numeric(dhs_ind$age_ew) - 4
dhs_ind$age_ew <- as.factor(dhs_ind$age_ew)
levels(dhs_ind$age_ew) <- levels(dhs_ind$age)[c(5:11)]

synthpop_ind_loc$age_ew <- synthpop_ind_loc$age
synthpop_ind_loc[synthpop_ind_loc$elig_women == 0,]$age_ew <- NA
synthpop_ind_loc$age_ew <- as.numeric(synthpop_ind_loc$age_ew) - 4
synthpop_ind_loc$age_ew <- as.factor(synthpop_ind_loc$age_ew)
levels(synthpop_ind_loc$age_ew) <- levels(synthpop_ind_loc$age)[c(5:11)]

#use middle age category as reference cat
dhs_ind$age_ew <- relevel(dhs_ind$age_ew, ref = "30 - 34")

model_contraception <- multinom(contraception ~ urban_rural + educ_hhhead +
                                                toilet + water + space + structure + fuel +
                                                age_ew 
                                ,data = dhs_ind, weights = hh_weight, subset = (elig_women == 1))

mod_stargazer("output/populations/synth_pop_1/contraception.txt", model_contraception, header=FALSE, type = "text")

loglik_model_contraception <- nnet:::logLik.multinom(model_contraception)
model_contraception0 <- multinom(contraception ~ 1, data = dhs_ind, weights = hh_weight, subset = (elig_women == 1))
loglik_model_contraception0 <- nnet:::logLik.multinom(model_contraception0)
mfr2_model_contraception <- as.numeric(1 - loglik_model_contraception/loglik_model_contraception0)
mfr2_model_contraception

synthpop_ind_loc$contraception <- predictMNL(model_contraception,synthpop_ind_loc)
synthpop_ind_loc[synthpop_ind_loc$elig_women == 0 ,]$contraception <- NA

prop.table(table(synthpop_ind_loc$contraception))
prop.table(wtd.table(x = dhs_ind[dhs_ind$region == "oshikoto",]$contraception, weights = dhs_ind[dhs_ind$region == "oshikoto",]$hh_weight))

synthpop_ind_loc$contraception <- as.factor(synthpop_ind_loc$contraception)
levels(synthpop_ind_loc$contraception) <- c("modern", "no/traditional")  # with 2 options the order seems to interchange

## dpt3
dhs_ind$age_u5 <- dhs_ind$age
dhs_ind[dhs_ind$u5 == 0,]$age_u5 <- NA
dhs_ind$age_u5 <- as.numeric(dhs_ind$age_u5)
dhs_ind$age_u5 <- as.factor(dhs_ind$age_u5)
levels(dhs_ind$age_u5) <- levels(dhs_ind$age)[c(1:2)]

synthpop_ind_loc$age_u5 <- synthpop_ind_loc$age
synthpop_ind_loc[synthpop_ind_loc$u5 == 0,]$age_u5 <- NA
synthpop_ind_loc$age_u5 <- as.numeric(synthpop_ind_loc$age_u5)
synthpop_ind_loc$age_u5 <- as.factor(synthpop_ind_loc$age_u5)
levels(synthpop_ind_loc$age_u5) <- levels(synthpop_ind_loc$age)[c(1:2)]

model_dpt3 <- multinom(dpt3 ~ urban_rural + educ_hhhead +
                              toilet + water + space + structure + fuel +
                              sex + age_u5
                       ,data = dhs_ind, weights = hh_weight, subset = (u5 == 1))

mod_stargazer("output/populations/synth_pop_1/dpt3.txt", model_dpt3, header=FALSE, type = "text")

loglik_model_dpt3 <- nnet:::logLik.multinom(model_dpt3)
model_dpt30 <- multinom(dpt3 ~ 1, data = dhs_ind, weights = hh_weight, subset = (u5 == 1))
loglik_model_dpt30 <- nnet:::logLik.multinom(model_dpt30)
mfr2_model_dpt3 <- as.numeric(1 - loglik_model_dpt3/loglik_model_dpt30)
mfr2_model_dpt3

synthpop_ind_loc$dpt3 <- predictMNL(model_dpt3,synthpop_ind_loc)
synthpop_ind_loc$dpt3 <- ifelse(synthpop_ind_loc$age %in% c("0", "1 - 4"), synthpop_ind_loc$dpt3, NA)

prop.table(table(synthpop_ind_loc$dpt3))
prop.table(wtd.table(x = dhs_ind[dhs_ind$region == "oshikoto",]$dpt3, weights = dhs_ind[dhs_ind$region == "oshikoto",]$hh_weight))

synthpop_ind_loc$dpt3 <- as.factor(synthpop_ind_loc$dpt3)
levels(synthpop_ind_loc$dpt3) <- c("yes", "no")   # with 2 options the order seems to interchange

saveRDS(synthpop_ind_loc, file="Output/populations/synth_pop_1/nam_synth_person_complete.rds")
write.csv(synthpop_ind_loc, file="Output/populations/synth_pop_1/nam_synth_person_complete.csv")

sink()

save.image(file = "NAM_stepC8.RData")

