####
## Step 0.2 Generate min,max, and mean grids for Oshikoto
####

##### 0. initialize

library(rgdal)
library(rgeos)
library(raster)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

##### 1. load data

admin_grid <- raster("Data/Spatial Covariates/WorldPop cov/nam_grid_100m_ccidadminl0.tif")
NAM_region <- readOGR("Data/Census EA/Dana/Namibia Statistics Agency census files/BOUNDARIES/Regional_boundaries.shp")

# check
proj4string(admin_grid) == proj4string(NAM_region)

# Generate layer for Oshikoto region --> generate transformed spatial covariates only for Oshikoto region
# add 1km buffer, else some household locations in HH-2011 file fall outside the grid
NAM_region_1km <- gBuffer(NAM_region, byid = TRUE, width = 1/111, quadsegs = 5) 
Oshikoto_grid_1km <- mask(admin_grid, NAM_region_1km[NAM_region_1km$REGION == "Oshikoto",])

# Generate layer to trim the covariate files with for focal point calculations (to reduce memory use). 
# should contain Oshikoto + 5km so to be able to calculate the focal points also at the borders. 
# to be sure lets add 6km. 
NAM_region_6km <- gBuffer(NAM_region, byid = TRUE, width = 6/111, quadsegs = 5) 
Oshikoto_grid_6km <- mask(admin_grid, NAM_region_6km[NAM_region_6km$REGION == "Oshikoto",])

temp <- trim(Oshikoto_grid_6km)
Oshikoto_grid_1km <- crop(Oshikoto_grid_1km, extent(temp))

rm(admin_grid, NAM_region,temp)
gc()
  
#define focal weight, radius = number of cells spanning 5 km * res of cells. 
radius_fw <- (5000/100)*(res(Oshikoto_grid_1km)[1])
fw <- focalWeight(Oshikoto_grid_1km, radius_fw, type='circle') 
fw[fw>0] <- 1 

# trick focal into producing NA when all cells are NA 
# (should do so by itself according to help file, but instead throws warnings and produces inf),
# with this function that actually cannot be set to na.rm = FALSE
# not bullet proof, the max function still throws warnings, but output looks fine. 
my.min <- function(x, na.rm = TRUE) {
  ifelse(all(is.na(x)), NA, min(x, na.rm = TRUE))
}
my.max <- function(x, na.rm = TRUE) {
  ifelse(all(is.na(x)), NA, max(x, na.rm = TRUE))
}

## covariate data
cov_list <- list.files("Data/Spatial Covariates/WorldPop cov", pattern="\\.tif$")
exclude_cov <- paste0("nam_grid_100m_", c("ccidadminl0", "ccidadminl1",  #regional boundaries
                            "cciwat_cls", "level1_water",  #binary representation of inland waterbodies
                            "ghsl_guf_2012", "ghsl_guf_dst_2012", "guf_dst_2012", #(distance to) settlement buildup area's
                            "px_area"), ".tif") #gridcell area
cov_list <- cov_list[!cov_list %in% exclude_cov]
cov_list <- gsub("nam_grid_100m_","", cov_list, fixed=T)
cov_list <- gsub(".tif","", cov_list, fixed=T)

for(f in cov_list){
  print(f)
  
  grid <- raster(paste0("Data/Spatial Covariates/WorldPop cov/nam_grid_100m_", f, ".tif"))
  
  #scale just like in Step A_23 - spatial model.R
  mean_vals <- mean(values(grid), na.rm = TRUE)
  sd_vals <- sd(values(grid), na.rm = TRUE)

  grid <- mask(grid, Oshikoto_grid_6km)
  grid <- trim(grid)
  vals <- values(grid)
  vals <- (vals - mean_vals)/sd_vals
  grid <- setValues(grid, vals) # update grid
  
  rm(vals, mean_vals, sd_vals)
  gc()

  grid_mean <- focal(grid, w=fw, fun=mean , na.rm=TRUE)
  grid_mean <- mask(grid_mean, Oshikoto_grid_1km)
  grid_mean <- trim(grid_mean)
  writeRaster(grid_mean, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_mean.tif"), "GTiff")
  
  grid_min <- focal(grid, w=fw, fun=my.min, na.rm=TRUE)
  grid_min <- mask(grid_min, Oshikoto_grid_1km)
  grid_min <- trim(grid_min)
  writeRaster(grid_min, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_min.tif"), "GTiff")
  
  grid_max <- focal(grid, w=fw, fun=my.max, na.rm=TRUE)
  grid_max <- mask(grid_max, Oshikoto_grid_1km)
  grid_max <- trim(grid_max)
  writeRaster(grid_max, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_max.tif"), "GTiff")
  
  grid <- mask(grid, Oshikoto_grid_1km)
  grid <- trim(grid)
  writeRaster(grid, paste0("Data/Spatial Covariates/Generated/Oshikoto/only scaled/",f,".tif"), "GTiff")
  
  rm(grid, grid_mean, grid_min, grid_max)
  gc()
}

cov_list2 <- list.files("Data/Spatial Covariates/Generated", pattern="\\.tif$")
cov_list2 <- gsub("nam_grid_100m_","", cov_list2, fixed=T)
cov_list2 <- gsub(".tif","", cov_list2, fixed=T)

for(f in cov_list2){
  print(f)
  
  grid <- raster(paste0("Data/Spatial Covariates/Generated/nam_grid_100m_", f, ".tif"))
  
  #scale just like in Step 3
  mean_vals <- mean(values(grid), na.rm = TRUE)
  sd_vals <- sd(values(grid), na.rm = TRUE)
  
  grid <- mask(grid, Oshikoto_grid_6km)
  grid <- trim(grid)
  vals <- values(grid)
  vals <- (vals - mean_vals)/sd_vals
  grid <- setValues(grid, vals) # update grid
  
  rm(vals, mean_vals, sd_vals)
  gc()
  
  grid_mean <- focal(grid, w=fw, fun=mean , na.rm=TRUE)
  grid_mean <- mask(grid_mean, Oshikoto_grid_1km)
  grid_mean <- trim(grid_mean)
  writeRaster(grid_mean, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_mean.tif"), "GTiff")
  
  grid_min <- focal(grid, w=fw, fun=my.min, na.rm=TRUE)
  grid_min <- mask(grid_min, Oshikoto_grid_1km)
  grid_min <- trim(grid_min)
  writeRaster(grid_min, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_min.tif"), "GTiff")
  
  grid_max <- focal(grid, w=fw, fun=my.max, na.rm=TRUE)
  grid_max <- mask(grid_max, Oshikoto_grid_1km)
  grid_max <- trim(grid_max)
  writeRaster(grid_max, paste0("Data/Spatial Covariates/Generated/Oshikoto/",f,"_max.tif"), "GTiff")
  
  grid <- mask(grid, Oshikoto_grid_1km)
  grid <- trim(grid)
  writeRaster(grid, paste0("Data/Spatial Covariates/Generated/Oshikoto/only scaled/",f,".tif"), "GTiff")
  
  rm(grid, grid_mean, grid_min, grid_max)
  gc()
}
