####
## Phase A - step 1. Construct DHS cluster types
####

##### 0. initialize
library(foreign)
library(plyr)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

##### 1. load and merge data

## dhs ind recode file (household combined with women combined with child)
## pre-processed in Stata using Create_mom2_recode.do and create_ind_recode.do
dhs_ind <- read.dta("Data/DHS data/Generated data/DHS_ind_recode.dta")
summary(dhs_ind)

##### 2. from ind-level to hh-level

#hh_structure 
dhs_ind$spouse <- ifelse(dhs_ind$relationship=="spouse", 1, 0)
dhs_ind$child <- ifelse(dhs_ind$relationship %in% c("child"), 1, 0)
dhs_ind$gchild <- ifelse(dhs_ind$relationship=="gchild", 1, 0)
dhs_ind$extend <- ifelse(dhs_ind$relationship %in% c("extend"), 1, 0)
dhs_ind$other <- ifelse(dhs_ind$relationship %in% c("other"), 1, 0)
dhs_ind$u5 <- ifelse(dhs_ind$age %in% c("0", "1 - 4"), 1, 0)
dhs_ind <- ddply(dhs_ind,.(hhid), transform, 
                 any_spouse = max(spouse), 
                 any_child = max(child),
                 any_gchild = max(gchild),
                 any_extend = max(extend), 
                 any_other = max(other), 
                 any_u5 = max(u5))
                 
#select only hhead
dhs_hh <- dhs_ind[dhs_ind$relationship == "head",]
#drop individual level variables 
dhs_hh <- dhs_hh[, !names(dhs_hh) %in% c("age", "sex", "education", "spouse", "child", "gchild", "extend", "other", "u5")]

dhs_hh$noeduc <- ifelse(dhs_hh$educ_hhhead == "No formal", 1, 0) 
dhs_hh$noeduc[dhs_hh$educ_hhhead==6] <- NA
dhs_hh$higheduc <- ifelse(dhs_hh$educ_hhhead %in% c("Complete secondary", "Complete tertiary"), 1, 0) 
dhs_hh$higheduc[dhs_hh$educ_hhhead ==6] <- NA

dhs_hh$single <- ifelse(dhs_hh$hhsize == 1, 1, 0) 
dim(dhs_hh) # n=9849   30

##### 3. from hh-level to cluster-level

# aggregate to clusters - average
dhs_hh$urban_rural <- as.numeric(dhs_hh$urban_rural) - 1
# 1 is rural, 0 is urban
dhs_hh$water <- as.numeric(dhs_hh$water) - 1
dhs_hh$toilet <- as.numeric(dhs_hh$toilet) - 1
dhs_hh$space <- as.numeric(dhs_hh$space) - 1
dhs_hh$structure <- as.numeric(dhs_hh$structure) - 1
dhs_hh$slum <- as.numeric(dhs_hh$slum) - 1
dhs_hh$fuel <- as.numeric(dhs_hh$fuel) - 1

dhs_clust_nam <- aggregate(dhs_hh[,c("urban_rural", "noeduc", "higheduc", 
                                     "any_spouse", "any_child", "any_gchild", "any_extend", "any_other", "any_u5", 
                                     "water", "toilet", "space", "structure", "slum", "fuel")], 
                           by=list("cluster"=dhs_hh$cluster_nr), function (x) mean(x, na.rm=TRUE))
head(dhs_clust_nam)
dim(dhs_clust_nam) # n=550 16
summary(dhs_clust_nam)
#little variation in water
corr <- cor(dhs_clust_nam[,-1])
summary(corr[upper.tri(corr)])
corr
#toilet and slum high correlation #0.96
#fuel and higher education... #0.81

dhs_hh_osh <- dhs_hh[dhs_hh$region %in% c("oshikoto"),]
dhs_clust_osh <- aggregate(dhs_hh_osh[,c("urban_rural", "noeduc", "higheduc", 
                                          "any_spouse", "any_child", "any_gchild", "any_extend", "any_other", "any_u5", 
                                          "water", "toilet", "space", "structure", "slum", "fuel")], 
                           by = list("cluster"=dhs_hh_osh$cluster_nr), function (x) mean(x, na.rm=TRUE))
head(dhs_clust_osh)
dim(dhs_clust_osh) # n=38 16
summary(dhs_clust_osh)
corr <- cor(dhs_clust_osh[,-1])
summary(corr[upper.tri(corr)])
corr

## we will decide on a classification based on the data from the Oshikoto region.
dhs_clust <- dhs_clust_osh

## cluster types
# scree plot
wssplot <- function(data, nc=15, seed=1234){
  wss <- (nrow(data)-1)*sum(apply(data,2,var))
  for (i in 2:nc){
    set.seed(seed)
    wss[i] <- sum(kmeans(data, centers=i, iter.max=20)$withinss)
    }
  plot(1:nc, wss, type="b", xlab="Number of Clusters",
  ylab="Within groups sum of squares")
}
#

# NOTE: We don't center and scale covariates for kmeans clustering, 
# because all the variables can be interpreted as percentages. 
# The current range of variables is thus comparable and relevant. 

cluster_var <- c("urban_rural", "noeduc", "any_u5", "toilet", "water", "structure", "space", "fuel")

# check kmeans
wssplot(dhs_clust[,cluster_var], nc=36)
# 6 or 7 clusters may be reasonable..
# start with 20 clusters and regroup based on dendogram
k <- kmeans(dhs_clust[,cluster_var], centers=20, iter.max=20)
table(k$cluster)

k.hc <- hclust(dist(k$centers))
plot(k.hc, hang=-1, main="KHM DHS cluster types", xlab="")
# 7 clusters (height = 0.65), gives clearly interpretable groups
cutline <- 0.65
abline(h=cutline, col="red")

# cut/regroup the kmeans into 7 classes based on dendrogram
k.hc.cut <- cutree(k.hc, h=cutline) # cross-walk table
k.hc.cut <- unname(k.hc.cut) # drop labels

# fancy plot
library(dendextend)
labelColors <- c("red","orange","black","green","blue","purple","pink")
# cut dendrogram in 4 clusters
clusMember = cutree(k.hc, h=cutline)
# function to get color labels
colLab <- function(n) {
  if (is.leaf(n)) {
    a <- attributes(n)
    labCol <- labelColors[clusMember[which(names(clusMember)==a$label)]]
    attr(n, "nodePar") <- c(a$nodePar, col=labCol, lab.col=labCol)
  }
  n
}
# using dendrapply
clusDendro = dendrapply(as.dendrogram(k.hc), colLab)
clusDendro %>% set("leaves_pch", 16) %>%
  # make plot
  plot(main="KHM DHS cluster types" )
abline(h=cutline, col="red")
#

# update
dhs_clust$hhclass <- k.hc.cut[k$cluster]

# view summary stats
colMeans(dhs_clust[,cluster_var])
by(dhs_clust[,cluster_var], dhs_clust$hhclass, FUN=colMeans)

saveRDS(k, file="output/nam_dhs_kmeans.rds")
saveRDS(k.hc, file="output/nam_dhs_cutkmeans.rds")

### Attach the cluster types based on Oshikoto data to all clusters in DHS Namibia

library(FNN)

dhs_clust <- dhs_clust_nam
km <- readRDS("Output/nam_dhs_kmeans.rds")
head(km)
pred.clust <- get.knnx(km$centers, dhs_clust[,cluster_var], 1)$nn.index[,1]
table(pred.clust)
prop.table(table(pred.clust))

# load reclassified
k.hc <- readRDS("Output/nam_dhs_cutkmeans.rds")
plot(k.hc, hang=-1)

# cut/regroup the kmeans into 5 classes based on dendrogram
k.hc.cut <- cutree(k.hc, h=cutline) # cross-walk table
k.hc.cut <- unname(k.hc.cut) # drop labels

# apply to data with reclassified clusters
dhs_clust$hhclass <- k.hc.cut[pred.clust]

## output cluster types
saveRDS(dhs_clust[,c("cluster","urban_rural","hhclass")], file= "output/nam_dhs_clustertypes.rds")
write.csv(dhs_clust[,c("cluster","urban_rural","hhclass")], file= "output/nam_dhs_clustertypes.csv", row.names=F)

save.image(file = "NAM_stepA1.RData")
