####
## Phase A - step 2 and 3. Predict grid types
####

##### 0. initialize

library(rgdal)
library(rgeos)
library(raster)
library(devtools)

library(randomForest)
library(caret)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

##### 1. load data

# DHS hh types
nam_dhs_clust <- readRDS("output/nam_dhs_clustertypes.rds")
dim(nam_dhs_clust) # n=550
head(nam_dhs_clust)
table(nam_dhs_clust$hhclass)

# load GPS data
dhs_gps_points <- readOGR("Data/DHS data/Original data/NM_2013_DHS_01022018_geo/nmge61fl/NMGE61FL.shp")
# generate a buffer with a radius of 5 kilometers
dhs_gps <- gBuffer(dhs_gps_points, byid = TRUE, width = 5/111)
rm(dhs_gps_points)

table(dhs_gps$SOURCE)
#no missings
names(dhs_gps) <- tolower(names(dhs_gps))

# drop unneeded variables
dhs_gps <- dhs_gps[,c("dhsid","dhsclust","latnum","longnum")]
summary(dhs_gps)
plot(dhs_gps)

## covariate data (files already downloaded)
cov_list <- list.files("Data/Spatial Covariates/WorldPop cov", pattern="\\.tif$")
exclude_cov <- paste0("nam_grid_100m_", c("ccidadminl0", "ccidadminl1",  #regional boundaries
                                          "cciwat_cls", "level1_water",  #binary representation of inland waterbodies
                                          "ghsl_guf_2012", "ghsl_guf_dst_2012", "guf_dst_2012", #(distance to) settlement buildup area's
                                          "px_area"), ".tif") #gridcell area
cov_list <- cov_list[!cov_list %in% exclude_cov]

admin_grid <- raster("Data/Spatial Covariates/WorldPop cov/nam_grid_100m_ccidadminl0.tif")
uid <- admin_grid
# unique cell ID
values(uid) <- 1:ncell(admin_grid)
uid <- mask(uid, admin_grid)
# extract cell IDs
cids <- extract(uid, dhs_gps)

head(dhs_gps)
# data store
return_dat <- dhs_gps$dhsclust
rm(uid, admin_grid, dhs_gps)

# extract value at cluster locations + 5km buffer
gc()
for(f in cov_list){
  print(f)
  
  grid <- raster(paste0("Data/Spatial Covariates/WorldPop cov/", f))
  vals <- values(grid)
  # vals_sc <- scale(vals, center=T, scale=T) 
  #takes up so much memory that vector cannot be allocated --> do it manually
  mean_vals <- mean(vals, na.rm = TRUE)
  sd_vals <- sd(vals, na.rm = TRUE)
  vals <- (vals - mean_vals)/sd_vals
  grid_scale <- setValues(grid, vals) # update grid
  # extract values
  svy_exvals <- lapply(cids, FUN=function(x){ extract(grid_scale, x) } ) # use cell numbers
  # summarize the extracted values
  # cast to numeric to avoid integer overflow problems
  svy_exvals <- lapply(svy_exvals, FUN=function(x){ data.frame("mean"=mean(as.numeric(x), na.rm=T),
                                                               "min"=min(as.numeric(x), na.rm=T),
                                                               "max"=max(as.numeric(x), na.rm=T),
                                                               "sd"=sd(as.numeric(x), na.rm=T)) })
  all_dat <- do.call(rbind, svy_exvals) # make a single item
  names(all_dat) <- paste0(f, "_", names(all_dat))
  return_dat <- cbind(return_dat, all_dat)
  rm(grid, grid_scale, vals, svy_exvals, mean_vals, sd_vals)
  gc()
}

cov_list2 <- list.files("Data/Spatial Covariates/Generated", pattern="\\.tif$")

for(f in cov_list2){
  print(f)
  
  grid <- raster(paste0("Data/Spatial Covariates/Generated/", f))
  vals <- values(grid)
  #vals_sc <- scale(vals, center=T, scale=T) 
  #takes up so much memory that vector cannot be allocated --> do it manually
  mean_vals <- mean(vals, na.rm = TRUE)
  sd_vals <- sd(vals, na.rm = TRUE)
  vals <- (vals - mean_vals)/sd_vals
  grid_scale <- setValues(grid, vals) # update grid
  # extract values
  svy_exvals <- lapply(cids, FUN=function(x){ extract(grid_scale, x) } ) # use cell numbers
  # summarize the extracted values
  # cast to numeric to avoid integer overflow problems
  svy_exvals <- lapply(svy_exvals, FUN=function(x){ data.frame("mean"=mean(as.numeric(x), na.rm=T),
                                                               "min"=min(as.numeric(x), na.rm=T),
                                                               "max"=max(as.numeric(x), na.rm=T),
                                                               "sd"=sd(as.numeric(x), na.rm=T)) })
  all_dat <- do.call(rbind, svy_exvals) # make a single item
  names(all_dat) <- paste0(f, "_", names(all_dat))
  return_dat <- cbind(return_dat, all_dat)
  rm(grid, grid_scale, vals, svy_exvals, mean_vals, sd_vals)
  gc()
}

names(return_dat)[1] <- "dhsclust"
names(return_dat) <- gsub("nam_grid_100m_","", names(return_dat), fixed=T)
names(return_dat) <- gsub(".tif","", names(return_dat), fixed=T)

# merge covariates back to dhs with hh type info
dhs_cov <- merge(nam_dhs_clust[,c("cluster","hhclass")], return_dat, by.x="cluster", by.y="dhsclust", all.x=T)
dim(dhs_cov) # 550 74
summary(dhs_cov)

# save file
saveRDS(dhs_cov, file="output/nam_dhs_spcovdata.rds")
#dhs_cov <- readRDS(file="Output/nam_dhs_spcovdata.rds")


##### 2. spatial model of HH type
# drop unneeded files
rm(cids)

fields <- names(dhs_cov)
# drop Std. Dev. stats - strange calculations
fields <- fields[!grepl("_sd",fields, fixed=T)]
fields <- fields[3:length(fields)]

# preprocessing
# drop near zero variance
# nzv <- nearZeroVar(dhs_cov[,3:ncol(dhs_cov)])
nzv <- nearZeroVar(dhs_cov[,fields])
nzv
####### watch out! If nzv empty this will not work ##### 
fields <- fields[-nzv]
dhs_cov <- dhs_cov[,c("cluster","hhclass", fields)]
# drop highly correlated
corr <- cor(dhs_cov[,fields])
summary(corr[upper.tri(corr)])
hicorr <- findCorrelation(corr, cutoff=.9)
hicorr
fields <- fields[-hicorr]
dhs_cov <- dhs_cov[,c("cluster","hhclass", fields)]
# check
corr <- cor(dhs_cov[,fields])
corr
summary(corr[upper.tri(corr)])
#
head(dhs_cov)
# tuning
x <- dhs_cov[,fields]
y <- as.factor(dhs_cov$hhclass)

initmod <- tuneRF(x=x, 
                  y=y, 
                  plot=TRUE, 
                  mtryStart=round(sqrt(ncol(x))), 
                  ntreeTry=1000, 
                  improve=0.00001, 
                  stepFactor=1.20,
                  trace=TRUE, doBest=TRUE, 
                  na.action=na.omit, 
                  importance=TRUE, 
                  proximity=FALSE, 
                  replace=TRUE
)
initmod

importance_scores <- importance(initmod)[order(importance(initmod)[,1], decreasing=TRUE),]
pos_importance <- rownames(importance_scores)[importance_scores[,1] > 0]

while (length(pos_importance) < length(importance_scores[,1])) {
  ##	Subset our x_data to just those columns having positive scores:
  x_data <- x[pos_importance]
  fitmod = tuneRF(x=x_data, 
                  y=y, 
                  plot=TRUE, 
                  mtryStart=round(sqrt(ncol(x))), 
                  ntreeTry=1000, 
                  improve=0.00001, 
                  stepFactor=1.20, 
                  trace=TRUE, doBest=TRUE, 
                  na.action=na.omit, 
                  importance=TRUE, 
                  proximity=FALSE, 
                  replace=TRUE
  ) 
  
  ##	Re-check importance scores:
  importance_scores <- importance(fitmod)[order(importance(fitmod)[,1], decreasing=TRUE),]
  pos_importance <- rownames(importance_scores)[importance_scores[,1] > 0]
  print(fitmod)
}

print(fitmod)
png(filename="Output/probs/importance.png")
varImpPlot(fitmod)
dev.off()

set.seed(1126)

nam_mod_final <- randomForest(x=x_data, y=y, mtry=fitmod$mtry, ntree=1000, proximity=T, replace=T)
nam_mod_final

# save object
saveRDS(nam_mod_final, file="Output/probs/nam_rf_mod.rds")
#nam_mod_final <- readRDS(file="Output/probs/nam_rf_mod.rds")


##### 3. spatial predictions

# load the processed grids 
covstack <- stack(list.files("Data/Spatial Covariates/Generated/Oshikoto", pattern=".tif$", full.names=T))
covstack <- subset(covstack, row.names(nam_mod_final$importance)) # subset for final list of variables
nlayers(covstack)

# prediction into multilayer raster
nam_probs <- predict(covstack, model=nam_mod_final, type="prob", index=1:7, filename="Output/probs/nam_type_probs.tif", progress="text", overwrite = TRUE)
# one layer per type, probabilities sum to 1 in a cell across all types

# generate a figure of the probability layers and check if the figure makes sense.  
library(rasterVis)
png(filename=paste0("Output/probs/nam_prob_surfaces.png"))
levelplot(nam_probs)
dev.off()

save.image(file = "NAM_stepA23.RData")
