####
## Phase B - Step 7. Assign synthetic population to household point locations
####

##### 0. initialize

library(raster)
library(foreign)
library(rgdal)
library(plyr)

setwd("C:/Users/lieke/Desktop/Proefschrift/Gridded sampling")

set.seed(1) #For population 1 we use set.seed(1), for population 2 we use set.seed(2) etc. 

##### 1. load and merge data

# load synthetic population household head file - remember: each population stored in seperate folder "synth_pop_x"

synth_pop_hh <- readRDS(file="Output/populations/synth_pop_1/nam_synth_hh.rds")
dim(synth_pop_hh) # 37299    30
str(synth_pop_hh)
synth_pop_hh$urban_rural <- as.factor(synth_pop_hh$urban_rural)
levels(synth_pop_hh$urban_rural) <- c("Urban", "Rural")

table(synth_pop_hh$hhclass, synth_pop_hh$urban_rural)
#some urban individuals classified as rural rich/rural poor --> change those to urban rich/rural poor
##NOTE: check if indeed type 4 is urban rich, type 5 is urban poor, this can be rearranged when rerunning
synth_pop_hh[(synth_pop_hh$hhclass == 1 & synth_pop_hh$urban_rural == "Urban"),]$hhclass <- 4
synth_pop_hh[(synth_pop_hh$hhclass == 2 & synth_pop_hh$urban_rural == "Urban"),]$hhclass <- 5
#check
table(synth_pop_hh$hhclass, synth_pop_hh$urban_rural)

# load household point location data
location_hh <- readOGR("Data/HH-2011_oshikoto_dwell_v4/HH-2011_oshikoto_dwell_v4.shp")
location_hh <- location_hh[, names(location_hh) %in% c("long_act", "lat_act", "ID_new", "EA", "TYPE", "Constitu_1", "Region")]

# long_act and lat_act missing 10 times in data.frame, take long/lat from coords for these observations
location_hh$long_act <- ifelse(location_hh$long_act == 0, location_hh@coords[,1], location_hh$long_act)
location_hh$lat_act <- ifelse(location_hh$lat_act == 0, location_hh@coords[,2], location_hh$lat_act)

#constituency, EA, and TYPE missing twice.
NAM_const <- readOGR("Data/Census EA/Dana/Namibia Statistics Agency census files/BOUNDARIES/Constituency_boundaries.shp")
NAM_const <- NAM_const[NAM_const$REGION %in% "Oshikoto",]
location_hh[!is.na(location_hh$TYPE),]@coords
over(location_hh[(location_hh$lat_act > -17.92330 & location_hh$lat_act < -17.92220 & location_hh$long_act < 16),], NAM_const)
#suggests that we can attach constituency name Olukonda (constitu_ = 4) to it, and thus TYPE Rural

NAM_EA <- readOGR("Data/Census EA/Dana/Namibia Statistics Agency census files/CENSUS EAs_2011/EA_2011_12June12.shp")
NAM_EA <- NAM_EA[NAM_EA$Region_Nam == "Oshikoto",]
over(location_hh[(location_hh$lat_act > -17.92330 & location_hh$lat_act < -17.92220 & location_hh$long_act < 16),], NAM_EA)
#suggests that we can attach EA name 120499001 (EA = 147) to it

location_hh$Constitu_1 <- ifelse(is.na(location_hh$Constitu_1), 4, location_hh$Constitu_1)
location_hh$Constitu_1 <- as.factor(location_hh$Constitu_1)
levels(location_hh$Constitu_1) <- c("Engodi", "Guinas", "Okankolo", "Olukonda", "Omuntele", 
                                    "Omuthiyagwipundi", "Onayena", "Oniipa","Onyaanya","Tsumeb")          
location_hh@data[location_hh@data$Constitu_1 %in% c("Guinas", "Olukonda", "Oniipa"),]$TYPE <- "Rural"
location_hh@data[location_hh@data$Constitu_1 %in% c("Tsumeb"),]$TYPE <- "Urban"
temp <- levels(location_hh$EA)
location_hh$EA <- ifelse(is.na(location_hh$EA), 147, location_hh$EA)
location_hh$EA <- as.factor(location_hh$EA)
levels(location_hh$EA) <- temp
rm(temp)

location_hh$uniq_id <- c(1:nrow(location_hh))

# hhtype probabilities
probs <- stack("Output/probs/nam_type_probs.tif")
probs

# probs rich/poor devision trough visual inspection of urban area's 
EA_urbpoor <- readOGR("Data/ea_2011_v10_oshikoto/ea_2011_v10_oshikoto.shp")
EA_urbpoor <- EA_urbpoor[,names(EA_urbpoor) %in% c("EA", "urb_poor")]

##### 2. Assign probabilities to locations

location_hh$grid_nr <- cellFromXY(probs[[1]], location_hh@coords)

location_hh$prob_1 <-  probs[[1]][location_hh$grid_nr]
location_hh$prob_2 <-  probs[[2]][location_hh$grid_nr]
location_hh$prob_3 <-  probs[[3]][location_hh$grid_nr]
location_hh$prob_4 <-  probs[[4]][location_hh$grid_nr]
location_hh$prob_5 <-  probs[[5]][location_hh$grid_nr]
location_hh$prob_6 <-  probs[[6]][location_hh$grid_nr]
location_hh$prob_7 <-  probs[[7]][location_hh$grid_nr]

##We adjust the probabilities of type 4 and 5 the urban types to probabilities 
## estimated by visual inspection of satellite imagery (Phase A, step 4) 
##(make sure to merge and then match rather than only merge, else the order gets messed up.)
##NOTE: check if indeed type 4 is urban rich, type 5 is urban poor, this can be rearranged when rerunning
temp <- ddply(merge(location_hh@data, EA_urbpoor@data), .(EA))
location_hh@data = data.frame(location_hh@data, temp[match(location_hh@data$uniq_id, temp$uniq_id),])
rm(temp)

location_hh$prob_4 <- location_hh$prob_4*(1-location_hh$urb_poor)
location_hh$prob_5 <- location_hh$prob_5*location_hh$urb_poor

#When only locations with prob 0 remain it is not possible to sample, so change to a very small probability. 
location_hh$prob_1[location_hh$prob_1 == 0] <- 0.00001
location_hh$prob_2[location_hh$prob_2 == 0] <- 0.00001
location_hh$prob_3[location_hh$prob_3 == 0] <- 0.00001
location_hh$prob_4[location_hh$prob_4 == 0] <- 0.00001
location_hh$prob_5[location_hh$prob_5 == 0] <- 0.00001
location_hh$prob_6[location_hh$prob_6 == 0] <- 0.00001
location_hh$prob_7[location_hh$prob_7 == 0] <- 0.00001

location_hh <- location_hh[!is.na(location_hh$prob_1),]

##### 3. Assign households to locations

hh_loc_match <- data.frame(hhid = integer(), uniq_id = integer())

# per constituency/urban-rural
for (constit in levels(synth_pop_hh$constituency)) {
  for (type_ur in levels(synth_pop_hh$urban_rural)) {
    alloc_hh <- synth_pop_hh[names(synth_pop_hh) %in% c("hhid", "constituency", "urban_rural", "hhclass")] # households still to allocate to an individual
    alloc_loc <- location_hh@data #locations still to allocate to a household
    alloc_loc <- alloc_loc[names(alloc_loc) %in% c("uniq_id", "Constitu_1", "TYPE", "prob_1", "prob_2", "prob_3", "prob_4",
                                                   "prob_5", "prob_6", "prob_7")]
    
    alloc_hh <- alloc_hh[(alloc_hh$urban_rural == type_ur & alloc_hh$constituency == constit),]
    alloc_loc <- alloc_loc[(alloc_loc$TYPE == type_ur & alloc_loc$Constitu_1 == constit),]

    if (nrow(alloc_hh) > 0 & nrow(alloc_loc) > 0) {
      print(paste0("constituency: ", constit))
      print(paste0("type: ", type_ur))      
      print(table(alloc_hh$hhclass))
      
      #if too many households, delete some at random
      if (nrow(alloc_hh) > nrow(alloc_loc)) {
        size_diff <- nrow(alloc_hh) - nrow(alloc_loc)
        print(paste0(size_diff, " households deleted"))
        hh_del <- sample(nrow(alloc_hh), size_diff,replace = FALSE)
        print(paste0("from ", nrow(alloc_hh)))
        alloc_hh <- alloc_hh[-hh_del,]
        print(paste0("to ", nrow(alloc_hh)))
        rm(hh_del)
      }
      #if too many locations, delete some at random (should never happen!)
      if (nrow(alloc_hh) < nrow(alloc_loc)) {
        size_diff <- nrow(alloc_loc) - nrow(alloc_hh)
        print(paste0(size_diff, " locations deleted"))
        loc_del <- sample(nrow(alloc_loc), size_diff,replace = FALSE)
        print(paste0("from ", nrow(alloc_loc)))
        alloc_loc <- alloc_loc[-loc_del,]
        print(paste0("to ", nrow(alloc_loc)))
        rm(loc_del)
      } 
      
      # assign each household to a location
      hh_match <- vector(length = nrow(alloc_hh))
      loc_match <- vector(length = nrow(alloc_hh))
      
      for (i in c(1:nrow(alloc_hh))) {
        hh_sel <- sample(nrow(alloc_hh), 1)
        prob_sel <- paste0("prob_", alloc_hh[hh_sel,]$hhclass)
        loc_sel <- sample(nrow(alloc_loc), 1, prob = alloc_loc[,names(alloc_loc) %in% prob_sel])
        hh_match[i] <- alloc_hh[hh_sel,]$hhid
        loc_match[i] <- alloc_loc[loc_sel,]$uniq_id
        
        alloc_hh <- alloc_hh[-hh_sel,]
        alloc_loc <- alloc_loc[-loc_sel,]
      }
      temp <- data.frame(hh_match, loc_match)
      names(temp) <- c("hhid", "uniq_id")
      hh_loc_match <- rbind(hh_loc_match, temp)
      
      rm(alloc_hh, alloc_loc,loc_match, loc_sel,hh_match, hh_sel, prob_sel, size_diff,i, temp)
    }
  }
}

synth_pop_hh <- ddply(merge(synth_pop_hh, hh_loc_match), .(hhid))
synth_pop_hh <- synth_pop_hh[!is.na(synth_pop_hh$uniq_id),]

location_hh@data = data.frame(location_hh@data, synth_pop_hh[match(location_hh@data$uniq_id, synth_pop_hh$uniq_id),])
location_hh <- location_hh[!is.na(location_hh$urban_rural),]

#save file to disk
writeOGR(location_hh, dsn = "Output/populations/synth_pop_1", 
          layer = "Oshikoto_synthpop_loc", driver = "ESRI Shapefile", overwrite_layer = TRUE)

save.image(file = "NAM_stepB7.RData")

